@@ -486,20 +486,38 @@ class CTETest(fixtures.TestBase, AssertsCompiledSQL):
             "SELECT cs1.x, cs2.x AS x_1 FROM bar AS cs1, cte AS cs2",
         )
 
-    def test_conflicting_names(self):
+    @testing.combinations(True, False, argnames="identical")
+    @testing.combinations(True, False, argnames="use_clone")
+    def test_conflicting_names(self, identical, use_clone):
         """test a flat out name conflict."""
 
         s1 = select(1)
         c1 = s1.cte(name="cte1", recursive=True)
-        s2 = select(1)
-        c2 = s2.cte(name="cte1", recursive=True)
+        if use_clone:
+            c2 = c1._clone()
+            if not identical:
+                c2 = c2.union(select(2))
+        else:
+            if identical:
+                s2 = select(1)
+            else:
+                s2 = select(column("q"))
+            c2 = s2.cte(name="cte1", recursive=True)
 
         s = select(c1, c2)
-        assert_raises_message(
-            CompileError,
-            "Multiple, unrelated CTEs found " "with the same name: 'cte1'",
-            s.compile,
-        )
+
+        if use_clone and identical:
+            self.assert_compile(
+                s,
+                'WITH RECURSIVE cte1("1") AS (SELECT 1) SELECT cte1.1, '
+                'cte1.1 AS "1_1" FROM cte1',
+            )
+        else:
+            assert_raises_message(
+                CompileError,
+                "Multiple, unrelated CTEs found " "with the same name: 'cte1'",
+                s.compile,
+            )
 
     def test_with_recursive_no_name_currently_buggy(self):
         s1 = select(1)
@@ -1613,6 +1631,72 @@ class CTETest(fixtures.TestBase, AssertsCompiledSQL):
             },
         )
 
+    def test_textual_select_uses_independent_cte_one(self):
+        """test #7760"""
+        products = table("products", column("id"), column("price"))
+
+        upd_cte = (
+            products.update().values(price=10).where(products.c.price > 50)
+        ).cte()
+
+        stmt = (
+            text(
+                "SELECT products.id, products.price "
+                "FROM products WHERE products.price < :price_2"
+            )
+            .columns(products.c.id, products.c.price)
+            .bindparams(price_2=45)
+            .add_cte(upd_cte)
+        )
+
+        self.assert_compile(
+            stmt,
+            "WITH anon_1 AS (UPDATE products SET price=:param_1 "
+            "WHERE products.price > :price_1) "
+            "SELECT products.id, products.price "
+            "FROM products WHERE products.price < :price_2",
+            checkparams={"param_1": 10, "price_1": 50, "price_2": 45},
+        )
+
+    def test_textual_select_uses_independent_cte_two(self):
+
+        foo = table("foo", column("id"))
+        bar = table("bar", column("id"), column("attr"), column("foo_id"))
+        s1 = select(foo.c.id)
+        s2 = text(
+            "SELECT bar.id, bar.attr FROM bar "
+            "WHERE bar.foo_id IN (SELECT id FROM baz)"
+        ).columns(bar.c.id, bar.c.attr)
+        s3 = s2.add_cte(s1.cte(name="baz"))
+
+        self.assert_compile(
+            s3,
+            "WITH baz AS (SELECT foo.id AS id FROM foo) "
+            "SELECT bar.id, bar.attr FROM bar WHERE bar.foo_id IN "
+            "(SELECT id FROM baz)",
+        )
+
+    def test_textual_select_stack_correction(self):
+        """test #7798 , regression from #7760"""
+
+        foo = table("foo", column("id"))
+        bar = table("bar", column("id"), column("attr"), column("foo_id"))
+
+        s1 = text("SELECT id FROM foo").columns(foo.c.id)
+        s2 = text(
+            "SELECT bar.id, bar.attr FROM bar WHERE br.id IN "
+            "(SELECT id FROM baz)"
+        ).columns(bar.c.id, bar.c.attr)
+        s3 = bar.insert().from_select(list(s2.selected_columns), s2)
+        s4 = s3.add_cte(s1.cte(name="baz"))
+
+        self.assert_compile(
+            s4,
+            "WITH baz AS (SELECT id FROM foo) INSERT INTO bar (id, attr) "
+            "SELECT bar.id, bar.attr FROM bar WHERE br.id IN "
+            "(SELECT id FROM baz)",
+        )
+
     def test_insert_uses_independent_cte(self):
         products = table("products", column("id"), column("price"))
 
@@ -2011,7 +2095,8 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             ") SELECT cte.outer_cte FROM cte",
         )
 
-    def test_nesting_cte_in_recursive_cte(self):
+    @testing.fixture
+    def nesting_cte_in_recursive_cte(self):
         nesting_cte = select(literal(1).label("inner_cte")).cte(
             "nesting", nesting=True
         )
@@ -2020,20 +2105,85 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             "rec_cte", recursive=True
         )
         rec_part = select(rec_cte.c.outer_cte).where(
-            rec_cte.c.outer_cte == literal(1)
+            rec_cte.c.outer_cte == literal(42)
         )
         rec_cte = rec_cte.union(rec_part)
 
         stmt = select(rec_cte)
+        return stmt
+
+    def test_nesting_cte_in_recursive_cte_positional(
+        self, nesting_cte_in_recursive_cte
+    ):
 
         self.assert_compile(
-            stmt,
+            nesting_cte_in_recursive_cte,
+            "WITH RECURSIVE rec_cte(outer_cte) AS (WITH nesting AS "
+            "(SELECT ? AS inner_cte) "
+            "SELECT nesting.inner_cte AS outer_cte FROM nesting UNION "
+            "SELECT rec_cte.outer_cte AS outer_cte FROM rec_cte "
+            "WHERE rec_cte.outer_cte = ?) "
+            "SELECT rec_cte.outer_cte FROM rec_cte",
+            checkpositional=(1, 42),
+            dialect="default_qmark",
+        )
+
+    def test_nesting_cte_in_recursive_cte(self, nesting_cte_in_recursive_cte):
+        self.assert_compile(
+            nesting_cte_in_recursive_cte,
             "WITH RECURSIVE rec_cte(outer_cte) AS (WITH nesting AS "
             "(SELECT :param_1 AS inner_cte) "
             "SELECT nesting.inner_cte AS outer_cte FROM nesting UNION "
             "SELECT rec_cte.outer_cte AS outer_cte FROM rec_cte "
             "WHERE rec_cte.outer_cte = :param_2) "
             "SELECT rec_cte.outer_cte FROM rec_cte",
+            checkparams={"param_1": 1, "param_2": 42},
+        )
+
+    @testing.fixture
+    def nesting_cte_in_recursive_cte_w_add_cte(self):
+        nesting_cte = select(literal(1).label("inner_cte")).cte(
+            "nesting", nesting=True
+        )
+
+        rec_cte = select(nesting_cte.c.inner_cte.label("outer_cte")).cte(
+            "rec_cte", recursive=True
+        )
+        rec_part = select(rec_cte.c.outer_cte).where(
+            rec_cte.c.outer_cte == literal(42)
+        )
+        rec_cte = rec_cte.union(rec_part)
+
+        stmt = select(rec_cte)
+        return stmt
+
+    def test_nesting_cte_in_recursive_cte_w_add_cte_positional(
+        self, nesting_cte_in_recursive_cte_w_add_cte
+    ):
+        self.assert_compile(
+            nesting_cte_in_recursive_cte_w_add_cte,
+            "WITH RECURSIVE rec_cte(outer_cte) AS (WITH nesting AS "
+            "(SELECT ? AS inner_cte) "
+            "SELECT nesting.inner_cte AS outer_cte FROM nesting UNION "
+            "SELECT rec_cte.outer_cte AS outer_cte FROM rec_cte "
+            "WHERE rec_cte.outer_cte = ?) "
+            "SELECT rec_cte.outer_cte FROM rec_cte",
+            checkpositional=(1, 42),
+            dialect="default_qmark",
+        )
+
+    def test_nesting_cte_in_recursive_cte_w_add_cte(
+        self, nesting_cte_in_recursive_cte_w_add_cte
+    ):
+        self.assert_compile(
+            nesting_cte_in_recursive_cte_w_add_cte,
+            "WITH RECURSIVE rec_cte(outer_cte) AS (WITH nesting AS "
+            "(SELECT :param_1 AS inner_cte) "
+            "SELECT nesting.inner_cte AS outer_cte FROM nesting UNION "
+            "SELECT rec_cte.outer_cte AS outer_cte FROM rec_cte "
+            "WHERE rec_cte.outer_cte = :param_2) "
+            "SELECT rec_cte.outer_cte FROM rec_cte",
+            checkparams={"param_1": 1, "param_2": 42},
         )
 
     def test_recursive_nesting_cte_in_cte(self):
@@ -2135,18 +2285,19 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             "SELECT cte.outer_cte FROM cte",
         )
 
-    def test_same_nested_cte_is_not_generated_twice(self):
+    @testing.fixture
+    def same_nested_cte_is_not_generated_twice(self):
         # Same = name and query
         nesting_cte_used_twice = select(literal(1).label("inner_cte_1")).cte(
             "nesting_cte", nesting=True
         )
         select_add_cte = select(
-            (nesting_cte_used_twice.c.inner_cte_1 + 1).label("next_value")
+            (nesting_cte_used_twice.c.inner_cte_1 + 2).label("next_value")
         ).cte("nesting_2", nesting=True)
 
         union_cte = (
             select(
-                (nesting_cte_used_twice.c.inner_cte_1 - 1).label("next_value")
+                (nesting_cte_used_twice.c.inner_cte_1 - 3).label("next_value")
             )
             .union(select(select_add_cte))
             .cte("wrapper", nesting=True)
@@ -2157,9 +2308,36 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             .add_cte(nesting_cte_used_twice)
             .union(select(nesting_cte_used_twice))
         )
+        return stmt
 
+    def test_same_nested_cte_is_not_generated_twice_positional(
+        self, same_nested_cte_is_not_generated_twice
+    ):
         self.assert_compile(
-            stmt,
+            same_nested_cte_is_not_generated_twice,
+            "WITH nesting_cte AS "
+            "(SELECT ? AS inner_cte_1)"
+            ", wrapper AS "
+            "(WITH nesting_2 AS "
+            "(SELECT nesting_cte.inner_cte_1 + ? "
+            "AS next_value "
+            "FROM nesting_cte)"
+            " SELECT nesting_cte.inner_cte_1 - ? "
+            "AS next_value "
+            "FROM nesting_cte UNION SELECT nesting_2.next_value "
+            "AS next_value FROM nesting_2)"
+            " SELECT wrapper.next_value "
+            "FROM wrapper UNION SELECT nesting_cte.inner_cte_1 "
+            "FROM nesting_cte",
+            checkpositional=(1, 2, 3),
+            dialect="default_qmark",
+        )
+
+    def test_same_nested_cte_is_not_generated_twice(
+        self, same_nested_cte_is_not_generated_twice
+    ):
+        self.assert_compile(
+            same_nested_cte_is_not_generated_twice,
             "WITH nesting_cte AS "
             "(SELECT :param_1 AS inner_cte_1)"
             ", wrapper AS "
@@ -2169,19 +2347,25 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             "FROM nesting_cte)"
             " SELECT nesting_cte.inner_cte_1 - :inner_cte_1_1 "
             "AS next_value "
-            "FROM nesting_cte UNION SELECT nesting_2.next_value AS next_value "
-            "FROM nesting_2)"
+            "FROM nesting_cte UNION SELECT nesting_2.next_value "
+            "AS next_value FROM nesting_2)"
             " SELECT wrapper.next_value "
             "FROM wrapper UNION SELECT nesting_cte.inner_cte_1 "
             "FROM nesting_cte",
+            checkparams={
+                "param_1": 1,
+                "inner_cte_1_2": 2,
+                "inner_cte_1_1": 3,
+            },
         )
 
-    def test_recursive_nesting_cte_in_recursive_cte(self):
+    @testing.fixture
+    def recursive_nesting_cte_in_recursive_cte(self):
         nesting_cte = select(literal(1).label("inner_cte")).cte(
             "nesting", nesting=True, recursive=True
         )
         nesting_rec_part = select(nesting_cte.c.inner_cte).where(
-            nesting_cte.c.inner_cte == literal(1)
+            nesting_cte.c.inner_cte == literal(2)
         )
         nesting_cte = nesting_cte.union(nesting_rec_part)
 
@@ -2189,14 +2373,37 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             "rec_cte", recursive=True
         )
         rec_part = select(rec_cte.c.outer_cte).where(
-            rec_cte.c.outer_cte == literal(1)
+            rec_cte.c.outer_cte == literal(3)
         )
         rec_cte = rec_cte.union(rec_part)
 
         stmt = select(rec_cte)
+        return stmt
+
+    def test_recursive_nesting_cte_in_recursive_cte_positional(
+        self, recursive_nesting_cte_in_recursive_cte
+    ):
 
         self.assert_compile(
-            stmt,
+            recursive_nesting_cte_in_recursive_cte,
+            "WITH RECURSIVE rec_cte(outer_cte) AS ("
+            "WITH RECURSIVE nesting(inner_cte) AS "
+            "(SELECT ? AS inner_cte UNION "
+            "SELECT nesting.inner_cte AS inner_cte FROM nesting "
+            "WHERE nesting.inner_cte = ?) "
+            "SELECT nesting.inner_cte AS outer_cte FROM nesting UNION "
+            "SELECT rec_cte.outer_cte AS outer_cte FROM rec_cte "
+            "WHERE rec_cte.outer_cte = ?) "
+            "SELECT rec_cte.outer_cte FROM rec_cte",
+            checkpositional=(1, 2, 3),
+            dialect="default_qmark",
+        )
+
+    def test_recursive_nesting_cte_in_recursive_cte(
+        self, recursive_nesting_cte_in_recursive_cte
+    ):
+        self.assert_compile(
+            recursive_nesting_cte_in_recursive_cte,
             "WITH RECURSIVE rec_cte(outer_cte) AS ("
             "WITH RECURSIVE nesting(inner_cte) AS "
             "(SELECT :param_1 AS inner_cte UNION "
@@ -2206,6 +2413,7 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             "SELECT rec_cte.outer_cte AS outer_cte FROM rec_cte "
             "WHERE rec_cte.outer_cte = :param_3) "
             "SELECT rec_cte.outer_cte FROM rec_cte",
+            checkparams={"param_1": 1, "param_2": 2, "param_3": 3},
         )
 
     def test_select_from_insert_cte_with_nesting(self):
@@ -2334,7 +2542,43 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             ") SELECT cte.outer_cte FROM cte",
         )
 
-    def test_recursive_cte_referenced_multiple_times_with_nesting_cte(self):
+    @testing.fixture
+    def cte_in_compound_select(self):
+        upper = select(literal(1).label("z"))
+
+        lower_a_cte = select(literal(2).label("x")).cte("xx", nesting=True)
+        lower_a = select(literal(3).label("y")).add_cte(lower_a_cte)
+        lower_b = select(literal(4).label("w"))
+
+        stmt = upper.union_all(lower_a.union_all(lower_b))
+        return stmt
+
+    def test_cte_in_compound_select_positional(self, cte_in_compound_select):
+        self.assert_compile(
+            cte_in_compound_select,
+            "SELECT ? AS z UNION ALL (WITH xx AS "
+            "(SELECT ? AS x) "
+            "SELECT ? AS y UNION ALL SELECT ? AS w)",
+            checkpositional=(1, 2, 3, 4),
+            dialect="default_qmark",
+        )
+
+    def test_cte_in_compound_select(self, cte_in_compound_select):
+        self.assert_compile(
+            cte_in_compound_select,
+            "SELECT :param_1 AS z UNION ALL (WITH xx AS "
+            "(SELECT :param_2 AS x) "
+            "SELECT :param_3 AS y UNION ALL SELECT :param_4 AS w)",
+            checkparams={
+                "param_1": 1,
+                "param_2": 2,
+                "param_3": 3,
+                "param_4": 4,
+            },
+        )
+
+    @testing.fixture
+    def recursive_cte_referenced_multiple_times_with_nesting_cte(self):
         rec_root = select(literal(1).label("the_value")).cte(
             "recursive_cte", recursive=True
         )
@@ -2347,7 +2591,7 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             exists(
                 select(rec_root_ref.c.the_value)
                 .where(rec_root_ref.c.the_value < 10)
-                .limit(1)
+                .limit(5)
             ).label("val")
         ).cte("should_continue", nesting=True)
 
@@ -2363,13 +2607,43 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
         rec_cte = rec_root.union_all(rec_part)
 
         stmt = rec_cte.select()
+        return stmt
 
+    def test_recursive_cte_referenced_multiple_times_with_nesting_cte_pos(
+        self, recursive_cte_referenced_multiple_times_with_nesting_cte
+    ):
         self.assert_compile(
-            stmt,
+            recursive_cte_referenced_multiple_times_with_nesting_cte,
+            "WITH RECURSIVE recursive_cte(the_value) AS ("
+            "SELECT ? AS the_value UNION ALL ("
+            "WITH allow_multiple_ref AS ("
+            "SELECT recursive_cte.the_value AS the_value "
+            "FROM recursive_cte)"
+            ", should_continue AS (SELECT EXISTS ("
+            "SELECT allow_multiple_ref.the_value FROM allow_multiple_ref"
+            " WHERE allow_multiple_ref.the_value < ?"
+            " LIMIT ?) AS val) "
+            "SELECT allow_multiple_ref.the_value * ? AS anon_1"
+            " FROM allow_multiple_ref, should_continue "
+            "WHERE should_continue.val != 1"
+            " UNION ALL SELECT allow_multiple_ref.the_value * ?"
+            " AS anon_2 FROM allow_multiple_ref, should_continue"
+            " WHERE should_continue.val != 1))"
+            " SELECT recursive_cte.the_value FROM recursive_cte",
+            checkpositional=(1, 10, 5, 2, 3),
+            dialect="default_qmark",
+        )
+
+    def test_recursive_cte_referenced_multiple_times_with_nesting_cte(
+        self, recursive_cte_referenced_multiple_times_with_nesting_cte
+    ):
+        self.assert_compile(
+            recursive_cte_referenced_multiple_times_with_nesting_cte,
             "WITH RECURSIVE recursive_cte(the_value) AS ("
             "SELECT :param_1 AS the_value UNION ALL ("
             "WITH allow_multiple_ref AS ("
-            "SELECT recursive_cte.the_value AS the_value FROM recursive_cte)"
+            "SELECT recursive_cte.the_value AS the_value "
+            "FROM recursive_cte)"
             ", should_continue AS (SELECT EXISTS ("
             "SELECT allow_multiple_ref.the_value FROM allow_multiple_ref"
             " WHERE allow_multiple_ref.the_value < :the_value_2"
@@ -2381,4 +2655,11 @@ class NestingCTETest(fixtures.TestBase, AssertsCompiledSQL):
             " AS anon_2 FROM allow_multiple_ref, should_continue"
             " WHERE should_continue.val != true))"
             " SELECT recursive_cte.the_value FROM recursive_cte",
+            checkparams={
+                "param_1": 1,
+                "param_2": 5,
+                "the_value_2": 10,
+                "the_value_1": 2,
+                "the_value_3": 3,
+            },
         )
